﻿using SequencionReader.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SequencionReader.ApproximationFunctions
{
    class LeastSquareMethodExp : IApproximationFunction
    {
        public int polynomialOrder { get; private set; }
        private double maximalReziduum;

        private double[] xValues;
        private double[] yValues;
        private double[] polynomial;

        public LeastSquareMethodExp(int startPolynomialOrder, double maximalReziduum)
        {
            this.polynomialOrder = startPolynomialOrder;
            this.maximalReziduum = maximalReziduum;
        }

        public double Compute(double xValue)
        {
            double polynomialResult = 0;
            for (int i = 0; i < this.polynomial.Length; i++)
                polynomialResult += this.polynomial[i] * Math.Pow(xValue, i);
            return polynomialResult;
        }

        public void DefinePolynomial(double[] xValues, double[] yValues)
        {
            this.xValues = xValues;
            this.yValues = yValues;

            do
            {
                this.polynomial = null;
                double[][] matrix = new double[this.polynomialOrder + 1][];

                for (int i = 0; i < matrix.Length; i++)
                {
                    matrix[i] = new double[this.polynomialOrder + 1];
                    for (int k = 0; k < matrix[i].Length; k++)
                    {
                        double fiResult = 0;
                        for (int xi = 0; xi < xValues.Length; xi++)
                        {
                            fiResult += Math.Pow(xValues[xi], k) * Math.Pow(xValues[xi], i);
                        }
                        matrix[i][k] = fiResult;
                    }
                }

                double[] resultMatrix = new double[this.polynomialOrder + 1];

                for (int i = 0; i < this.polynomialOrder + 1; i++)
                {
                    for (int xi = 0; xi < xValues.Length; xi++)
                    {
                        resultMatrix[i] += Math.Pow(xValues[xi], i) * yValues[xi];
                    }
                }

                matrix = MatrixOperations.InvertMatrix(matrix);
                this.polynomial = MatrixOperations.MatrixMultiply(matrix, resultMatrix);

                bool success = CompareReziduum();
                if (success) break;
                else this.polynomialOrder++;

            }
            while (true);
        }

        private bool CompareReziduum()
        {
            double[] reziduums = new double[this.yValues.Length]; 
            for (int j = 0; j < this.yValues.Length; j++)
            {
                double polynomialResult = 0;
                for (int i = 0; i < this.polynomial.Length; i++)
                    polynomialResult += this.polynomial[i] * Math.Pow(xValues[j],i);

                reziduums[j] = this.yValues[j] - polynomialResult;
            }

            double resultRezidium = 0;
            for (int i = 0; i < reziduums.Length; i++)
                resultRezidium += Math.Pow(reziduums[i], 2);

            return resultRezidium < this.maximalReziduum;
        }
    }
}
