﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SequencionReader.ApproximationFunctions
{
    public class NewtonPolynomial : IApproximationFunction
    {
        private List<double> xValues;
        private List<double> yValues;
        private double[][] processTable;

        public void DefinePolynomial(double[] xValues, double[] yValues)
        {
            this.xValues = xValues.ToList();
            this.yValues = yValues.ToList();
            processTable = null;

            InitProcessTable();
            ComputeProcessTable();
        }

        private void InitProcessTable()
        {
            this.processTable = new double[this.xValues.Count][];
            for (int i = 0; i < this.yValues.Count; i++)
            {
                this.processTable[i] = new double[i + 1];
                this.processTable[i][0] = this.yValues[i];
            }
        }

        private void ComputeProcessTable()
        {
            for (int k = 1; k < this.xValues.Count; k++)
            {
                int step = 0;
                for (int i = k; i < this.xValues.Count; i = k + step)
                {
                    this.processTable[i][k] = (this.processTable[i][k - 1] - this.processTable[i - 1][k - 1]) / (this.xValues[i] - this.xValues[i - k]);
                    step++;
                }
            }
        }

        public double Compute(double xValue)
        {
            double result = 0;
            for (int i = 0; i < this.processTable.Length; i++)
            {
                int k = this.processTable[i].Length - 1;
                double difference = this.processTable[i][k];
                result += PolynomialPartResult(i, difference, xValue);
            }
            this.xValues.Add(xValue);
            this.yValues.Add(result);
            return result;
        }

        public double Compute(double[] xValues, double[] yValues, double xValue)
        {
            DefinePolynomial(xValues, yValues);
            return Compute(xValue);
        }

        private double PolynomialPartResult(int iteration, double difference, double xValue)
        {
            double result = 1;
            for (int i = 0; i < iteration; i++)
            {
                result = result * (xValue - this.xValues[i]);
            }
            return result * difference;
        }
    }
}
