﻿using SequencionReader.ApproximationFunctions;
using SequencionReader.SequencionParsers;
using SequencionReader.SequencionTypes;
using System;

namespace SequencionReader
{
    class Program
    {
        static void Main(string[] args)
        {
            SequencionParser parser = new SequencionParser("\n", ',');
            double[][] sequencions = null;

            try
            {
                sequencions = parser.Load(args[0]);
            }
            catch (Exception e)
            {
                Console.WriteLine(string.Format("Parse exception: {0}", e));
                Console.ReadLine();
                return;
            }

            ISequencion[] resolvers = new ISequencion[]
            {
                new ConstantSequencion(),
                new AritmeticSequencion(),
                new GeometricSequencion(),
                new FibonacciSequencion(),
                //new GeneralSequencion(new LeastSquareMethodExp(2,0.00001)), //have same precision like Newton Polynomial
                new GeneralSequencion(new NewtonPolynomial()),
            };

            double[][] result = new double[sequencions.Length][];

            for (int i = 0; i < result.Length; i++)
            {
                try
                {
                    foreach (ISequencion resolver in resolvers)
                    {
                        if (resolver.Is(sequencions[i]))
                        {
                            result[i] = resolver.Compute(sequencions[i], 8);
                            break;
                        }
                    }
                }
                catch (Exception e)
                {
                    //Console.WriteLine(string.Format("Comute exception: {0}", e)); didn't write exception because of reading input
                }
            }

            string output = parser.GetString(result, "0,0,0,0,0,0,0,0\n");
            Console.Write(output);
        }
    }
}
