﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace SequencionReader.SequencionParsers
{
    public class SequencionParser
    {
        private string lineSeparator;
        private char numberSeparator;

        public SequencionParser(string lineSeparator, char numberSeparator)
        {
            this.lineSeparator = lineSeparator;
            this.numberSeparator = numberSeparator;
        }

        public double[][] Parse(string data)
        {
            string[] lines = data.Split(new string[] { this.lineSeparator }, StringSplitOptions.None);
            double[][] result = new double[lines.Length][];
            for (int j = 0; j < lines.Length; j++)
            {
                string cleanLine = Regex.Replace(lines[j], @"\s+", string.Empty);
                string[] sequencionS = cleanLine.Split(numberSeparator);
                List<double> resultPart = new List<double>();
                //result[j] = new double[sequencionS.Length];
                for (int i = 0; i < sequencionS.Length; i++)
                {
                    if (!string.IsNullOrWhiteSpace(sequencionS[i]))
                        resultPart.Add(Convert.ToDouble(sequencionS[i]));
                }
                result[j] = resultPart.ToArray();
            }
            return result;
        }

        public double[][] Load(string path)
        {
            string text = File.ReadAllText(path);
            return Parse(text);
        }

        public string GetString(double[][] sequencions, string nullSequencion)
        {
            StringBuilder result = new StringBuilder();

            for (int j = 0; j < sequencions.Length; j++)
            {
                if (sequencions[j] == null) result.Append(nullSequencion);
                for (int i = 0; i < sequencions[j].Length; i++)
                {
                    if (i != sequencions[j].Length - 1) result.AppendFormat("{0}{1}", sequencions[j][i], this.numberSeparator);
                    else result.AppendFormat("{0}{1}", sequencions[j][i], this.lineSeparator);
                }
            }

            return result.ToString();
        }

    }
}
